#include "MuonHitTest/CSCHitsValAlg.h"
#include "EventInfo/EventInfo.h"
#include "EventInfo/EventID.h" 

///for GaudiKernel
#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/AlgFactory.h"

#include "Identifier/Identifier.h"

#include "GeoAdaptors/GeoMuonHits.h"

#include "MuonGeoModel/MuonDetectorManager.h" 
#include "MuonGeoModel/CscReadoutElement.h"
 
#include "MuonSimEvent/CSCSimHitCollection.h"
#include "MuonSimEvent/CSCSimHit.h"

#include "MuonIdHelpers/CscIdHelper.h"
#include "MuonSimEvent/CscHitIdHelper.h"

#include "GeneratorObjects/McEventCollection.h"
#include "CLHEP/Vector/LorentzVector.h"

#include <TH2D.h>
#include "TTree.h"
#include <string>

using namespace MuonGM;

//void bookHistograms();
void bookCSCHistograms();
void cscbookTree();

CSCHitsValAlg::CSCHitsValAlg(const std::string& name, ISvcLocator* pSvcLocator) :
  Algorithm(name, pSvcLocator),
  m_pMuonMgr           (0),
  m_pCscIdHelper       (0),			
  ValAlgInterface(), 
  log( msgSvc(), name )
{
  // Set users' request
  /**Properties to be modified through the jobOption file
     Short description:
     Key for accessing MC truth
     Option to write out ntuple
     AANtuple file (linked to a specific file name in the jobOption)
     AANtuple directory name for CSC variables
     AANtuple tree name for CSC variables
     If true, enter the main algorithm loop
  */
  declareProperty("McEventKey",     m_key="TruthEvent");
  declareProperty("WriteNtuple",          m_writeNtuple = true);
  declareProperty("NtupleFileName",       m_ntupleFileName = "MuonHits");
  declareProperty("NtupleDirectoryName",  m_ntupleDirName = "CSCHitsValidation");
  declareProperty("NtupleTreeName",       m_ntupleTreeName = "CSCTestHits");
  declareProperty("DoCSCTest",m_DoCSCTest=true);   

  log << MSG::INFO << "++++++++++++  CSCHitsValAlg created  ++++++++++++"
      << endreq;
}

CSCHitsValAlg::~CSCHitsValAlg()
{
  log << MSG::INFO << " deleting CSCHitsValAlg " << endreq;
}

StatusCode CSCHitsValAlg::initialize()
{
  log << MSG::INFO << " initialize " << endreq;
  StatusCode status;

  /**Locate the StoreGateSvc and initialize our local ptr
     intitialize transient event store
  */
  status = service("StoreGateSvc", m_sgSvc);

  if (status.isFailure()) {
    log << MSG::FATAL << "CSCHitsValAlg: Could not retrieve StoreGateSvc!" << endreq;
    return status;
  }
  log << MSG::DEBUG << "Retrieved StoreGateSvc." << endreq;

  StoreGateSvc* detStore = 0;
  status = service( "DetectorStore", detStore );
  if ( status.isSuccess() ) 
    {
      status = detStore->retrieve( m_pMuonMgr );
      if ( status.isFailure() ) 
	{
	  log << MSG::ERROR 
	      << " CSCHitsValAlg::initialize() :  Could not find MuonDetDescrMgr " 
	      << endreq;
	} 
      else 
	{ 
	  m_pCscIdHelper = m_pMuonMgr->cscIdHelper();
	}
    } 
  else 
    {
      log << MSG::ERROR 
	  << " CSCHitsValAlg::initialize() : MuonDetDescrMgr not found in DetectorStore " 
	  << endreq;
    }    

      
  if (status.isFailure()){
    log <<MSG::ERROR << "Could not get DetectorStore"<<endreq;
    return( StatusCode::FAILURE );
  }
 
  bookCSCHistograms();

  /// ROOT AANtuple Initialization---------------------------------------

  if (m_writeNtuple) {

    std::string cscStreamAndPath = "/"+m_ntupleFileName+"/"+m_ntupleDirName;
    std::string csctreeName = m_ntupleTreeName;
 
    std::string csctreePath = cscStreamAndPath+"/"+csctreeName;

    csctree = new TTree(TString(csctreeName), "Muon CSC Hits output");
       
    StatusCode status;
    status=ToolRootHistSvc()->regTree(csctreePath, csctree);

    if(status.isFailure()) {
      log << MSG::DEBUG << "CSCHitsValAlg:: Unable to register TTreeTuple : " <<
	csctreePath << endreq;
      return status;   
    }

    //-----------------
    /**add items-variables to the validation AANTUPLE
       Short description:
       Total number of G4 hits in AANTUPLE
       Total number of events processed
       Type 1=Hits , 2=Digits , 3=Prepraw
       Run number, event specific ID
       Global X, Y, Z, R of hit (with respect to the interaction point)
       CSC station name (char or int), conventional ATLAS naming scheme. Range: [CSS, CSL]
       CSC station eta. Range: [-1,1] (-1 backward endcap, 1 forward endcap)
       CSC station phi, starts from station 1 and increases with phi. Range: [1,8] 
       CSC chamber layer, a layer of physical chambers, increases with |Z|. Range [1,2] (seems to be always 1 i hits)	
       CSC wire layer, a wire plane inside a chamber, increases with |Z|. Range [1,4] (two gas volumes of two wire layers each)
       CSC measures phi, info not available in G4 hits!
       CSC strip, info not available in G4 hits!    
    */  

    csctree->Branch("m_Validation_CSC_NumberOfHits",  &m_Validation_CSC_NumberOfHits, "m_Validation_CSC_NumberOfHits/I");
    csctree->Branch("m_Validation_CSC_EventNumber",  &m_Validation_CSC_EventNumber, "m_Validation_CSC_EventNumber/I");
    csctree->Branch("m_Validation_CSC_Type",  &m_Validation_CSC_Type, "m_Validation_CSC_Type/I");
    csctree->Branch("m_Validation_CSC_RunNumber",  &m_Validation_CSC_RunNumber, "m_Validation_CSC_RunNumber/I");	 
    csctree->Branch("m_Validation_CSC_HitX",  &m_Validation_CSC_HitX, "m_Validation_CSC_HitX/D"); 
    csctree->Branch("m_Validation_CSC_HitY",  &m_Validation_CSC_HitY, "m_Validation_CSC_HitY/D"); 
    csctree->Branch("m_Validation_CSC_HitZ",  &m_Validation_CSC_HitZ, "m_Validation_CSC_HitZ/D"); 
    csctree->Branch("m_Validation_CSC_HitR",  &m_Validation_CSC_HitR, "m_Validation_CSC_HitR/D");
 
    csctree->Branch("m_Validation_CSC_StationName",  &m_Validation_CSC_StationName, "m_Validation_CSC_StationName/C");
    csctree->Branch("m_Validation_CSC_StationEta",  &m_Validation_CSC_StationEta, "m_Validation_CSC_StationEta/I"); 
    csctree->Branch("m_Validation_CSC_StationPhi",  &m_Validation_CSC_StationPhi, "m_Validation_CSC_StationPhi/I"); 
    csctree->Branch("m_Validation_CSC_ChamberLayer",  &m_Validation_CSC_ChamberLayer, "m_Validation_CSC_ChamberLayer/I");
    csctree->Branch("m_Validation_CSC_WireLayer",  &m_Validation_CSC_WireLayer, "m_Validation_CSC_WireLayer/I");
    csctree->Branch("m_Validation_CSC_MeasuresPhi",  &m_Validation_CSC_MeasuresPhi, "m_Validation_CSC_MeasuresPhi/I"); 
    csctree->Branch("m_Validation_CSC_Strip",  &m_Validation_CSC_Strip, "m_Validation_CSC_Strip/I");  
    
  }





  log << MSG::INFO << "Initialisation ended     " << endreq;
  return StatusCode::SUCCESS;

}  

StatusCode CSCHitsValAlg::finalize()
{
  log << MSG::INFO << " finalize " << endreq;
	
  return StatusCode::SUCCESS;
}

/// Execute - on event by event

StatusCode CSCHitsValAlg::execute()
{

  log << MSG::INFO << " execute " << endreq;
  StatusCode sc;


  // ---------------------------------------
  // fill event data
  /** get EventInfo, used to obtain run and event number */
  const EventInfo* pevt;

  if (storeGateSvc()->retrieve(pevt).isFailure()) {
    log << MSG::FATAL << "Could not find event" << endreq;
    return(StatusCode::FAILURE);
  }
  else {
    log << MSG::INFO << "Found EventInfo in SG" << endreq;
  }	

  if(isVerbose()) 
    {
      log << MSG::VERBOSE <<"Processing EventInfo event #"<<pevt->event_ID()->event_number() << " run: " << pevt->event_ID()->run_number() << endreq;
    }


  /**Get number of events processed and run number*/
  int evt = pevt->event_ID()->event_number();
  int numrun = pevt->event_ID()->run_number();
  m_Validation_CSC_RunNumber = pevt->event_ID()->run_number();
	
  /**Access MC truth information*/
  const DataHandle<McEventCollection> mcEvent;

  if(storeGateSvc()->retrieve(mcEvent,m_key).isFailure())
    {
      log << MSG::FATAL << "CSCHitsValAlg: Could not find MCevent" << endreq;
      return StatusCode::SUCCESS;
    }
  else {
    log << MSG::INFO << "CSCHitsValAlg: Found MCEvent" << endreq;
  }	   
  

  DataVector<HepMC::GenEvent>::const_iterator e;
  if (mcEvent->size()!=1) return StatusCode::SUCCESS;
 
  HepVector3D direction;
  for (e=mcEvent->begin();e!=mcEvent->end(); e++) {
    for (HepMC::GenEvent::particle_const_iterator p= (**e).particles_begin(); p!= (**e).particles_end(); p++) 
      {
	  direction.setX((**p).momentum().px());
	  direction.setY((**p).momentum().py());
	  direction.setZ((**p).momentum().pz());
	  direction = direction.unit();
      }
  }



  /// ROOT histograms ---------------------------------------
  //MuonSpectrometer
  /**Get the generic MuonSpectrometer (MDT,RPC,CSC,TGC) histograms
     Short description:
     MuonSpectrometer cross section
     MuonSpectrometer longitudinal 
     Number of events processed from truth
     Event specific run number
     Eta, Theta, Phi, z residual (...), phi residual (...) of hits in MuonSpectrometer
  */
  TH2 *muondet;
  TH2 *muonlong_view;
  TH1 *event;
  TH1 *run;
  TH1 *eta;
  TH1 *theta;
  TH1 *phi;
  TH1 *zResid;
  TH1 *phiResid;

  sc=tHistSvc()->getHist("/muonhitpos/muondet_barrel",muondet);
  sc=tHistSvc()->getHist("/muonhitpos/muonlongitudinal",muonlong_view);
	
  sc=tHistSvc()->getHist("/truth/event_num",event);
  sc=tHistSvc()->getHist("/truth/run_num",run);
	
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_eta",eta);
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_theta",theta);
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_phi",phi);
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_zResid",zResid);
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_phiResid",phiResid);

  //CSC
  /**Get the CSC histograms
     Short description:
     MuonSpectrometer RPC cross section 
     MuonSpectrometer RPC longitudinal 
     Eta, Theta, Phi, z residual (...), phi residual (...) of hits in CSCs
  */ 
  TH2 *cscdet;
  TH2 *csclong_view; 
  TH1 *csctheta;
  TH1 *csceta;
  TH1 *cscphi;
  TH1 *csczResid;
  TH1 *cscphiResid;
		
  sc=tHistSvc()->getHist("/cschitpos/cschitpos_eta",csceta);
  sc=tHistSvc()->getHist("/cschitpos/cschitpos_theta",csctheta);
  sc=tHistSvc()->getHist("/cschitpos/cschitpos_phi",cscphi);
  sc=tHistSvc()->getHist("/cschitpos/cschitpos_zResid",csczResid);
  sc=tHistSvc()->getHist("/cschitpos/cschitpos_phiResid",cscphiResid);
  sc=tHistSvc()->getHist("/cschitpos/cscdet_barrel",cscdet);
  sc=tHistSvc()->getHist("/cschitpos/csclongitudinal",csclong_view);


  /**Enter the main algorithm loop*/
  if (m_DoCSCTest) {


    const DataHandle<CSCSimHitCollection> p_collection; 
    if (storeGateSvc()->retrieve(p_collection,"CSC_Hits")==StatusCode::FAILURE) 
      {
	log << MSG::FATAL << "No CSCSimHitCollection in StoreGate!" << endreq;
	return StatusCode::FAILURE;
      }
    else if(storeGateSvc()->retrieve(p_collection,"CSC_Hits")==StatusCode::SUCCESS)
      {
		
	/**Enter CSC hits loop, initialize the hits counter*/
	counter_ValHitNumber =0 ;
	for(CSCSimHitCollection::const_iterator i_hit=p_collection->begin(); i_hit!=p_collection->end(); ++i_hit) 
	  {
		
	    /**Increment hits counter*/		
	    ++counter_ValHitNumber;


	    /** Check the Hits identifiers, access the functions that give:
		Station name, station eta, station phi, chamber layer, wire layer.
		The values of these variables are written out to the AANtuple (variable content and range explained in the code section where AANTuple variables are
		registered)		
	    */				
	    HitID cschit= (*i_hit)->CSCid();	          
	    CscHitIdHelper* cschelper = CscHitIdHelper::GetHelper();
	    std::string csc_stname = cschelper->GetStationName(cschit);    
	    int         csc_steta  = cschelper->GetZSector(cschit);     
	    int         csc_stphi  = cschelper->GetPhiSector(cschit);
	    int         csc_cl     = cschelper->GetChamberLayer(cschit);
	    int         csc_wl     = cschelper->GetWireLayer(cschit);
	    int         csc_measphi = 1;
	    int         csc_strip =  1;

      
	    /**Create the offline identifiers, fill them with hit info.Ready to be accessed and retrieve info.
	       Currently not used in this code except for some checks.
	    */		            
			    
	    Identifier offid = m_pCscIdHelper->channelID(csc_stname, csc_steta, csc_stphi, csc_cl, csc_wl, csc_measphi, csc_strip);
	    if (offid == 0)
	      {
		log << MSG::FATAL << "CSC: Cannot build a valid Identifier; skip " << endreq;
	      }
	    log << MSG::INFO << " CSC: Offline Id "<<m_pCscIdHelper->show_to_string(offid) << endreq;

	    const MuonGM::CscReadoutElement* descriptor = m_pMuonMgr->getCscReadoutElement(offid);
	    if (descriptor != NULL)
	      {
		HepPoint3D xU;
		HepPoint3D localHit((*i_hit)->getHitStart().x(),(*i_hit)->getHitStart().y(),(*i_hit)->getHitStart().z());
		xU = descriptor->localToGlobalCoords(localHit,offid);
	      }
	    else
	      {
		log << MSG::FATAL << "CSC readout element not found for Id = " << m_pCscIdHelper->show_to_string(offid) << endreq;	 
		HepPoint3D(0,0,0);     
	      }
	    // -------- Testing Sim to Offline ID conversion ---------
     
	    HepPoint3D csc_gPos   = descriptor->stripPos(offid);
	    double csc_strip_phi   = csc_gPos.phi();
	    if (csc_strip_phi < 0.) csc_strip_phi += 2.*M_PI;
	    double csc_strip_perp  = csc_gPos.perp();
	    double csc_strip_z   = csc_gPos.z();
	    double csc_strip_cot   = 1./tan(csc_gPos.theta());


		 
	    //Check Hits 		
	    /**For every hit within the event, get the global position HepPoint3D u and then retrieve all releveant info
	       either from the HepPoint3D or from the MC vector (direction)
	    */				
	    GeoCSCHit ghit(**i_hit);
	    if (!ghit) continue;	
	    HepPoint3D u = ghit.getGlobalPosition();

	    event->Fill(evt);
	    run->Fill(numrun);		
		
	    muondet->Fill(u.x(),u.y());
	    cscdet->Fill(u.x(),u.y());		
	    double rad=sqrt(u.x()*u.x()+u.y()*u.y());

	    muonlong_view->Fill(u.z(),rad);
	    csclong_view->Fill(u.z(),rad);

	    theta->Fill(direction.theta());
	    csctheta->Fill(direction.theta());
				
	    eta->Fill(direction.eta());
	    csceta->Fill(direction.eta());

	    phi->Fill(direction.phi());
	    cscphi->Fill(direction.phi());
		
	    zResid->Fill(u.cross(direction).dot(direction.cross(HepVector3D(0,0,1)).unit()));
	    csczResid->Fill(u.cross(direction).z());

	    csczResid->Fill(u.cross(direction).dot(direction.cross(HepVector3D(0,0,1)).unit()));
	    cscphiResid->Fill(u.cross(direction).z()); 

	    /**AANtuple variables
	       Fill the AANtuple variables with a value obtained for each hit within the event
	    */		
	    m_Validation_CSC_Type = 1; //(1: Hit, 2: Digit, 3: Reco)
	    m_Validation_CSC_NumberOfHits = counter_ValHitNumber;
	    m_Validation_CSC_EventNumber=evt;;
	    m_Validation_CSC_HitX = u.x();
	    m_Validation_CSC_HitY = u.y();
	    m_Validation_CSC_HitZ = u.z();
	    m_Validation_CSC_HitR = rad;
		
	    strcpy(m_Validation_CSC_StationName,(csc_stname).c_str());  // put station name string in AANTUPLE as Val_StName  
	    m_Validation_CSC_StationEta = csc_steta;
	    m_Validation_CSC_StationPhi = csc_stphi;		
	    m_Validation_CSC_ChamberLayer = csc_cl;
	    m_Validation_CSC_WireLayer = csc_wl;
	    m_Validation_CSC_MeasuresPhi = csc_measphi;

	    /**After proccessing each hit, dump the info in the tree*/
	    csctree->Fill();
	  }/**Close the CSCSimHitCollection loop*/
		
      }/**Close part where CSCSimHitCollection is accessed from the StoreGate*/
	 
            
  }/**Close part of m_DoCSCTest*/

  return StatusCode::SUCCESS;    

}/**Close execute*/
/*---------------------------------------------------------*/
ITHistSvc* CSCHitsValAlg::ToolRootHistSvc()
  /*---------------------------------------------------------*/
{
  MsgStream log(msgSvc(), name());
  
  StatusCode sc = service("THistSvc",m_rootsvc, true);
  if( sc.isFailure() ) {
    log << MSG::WARNING << ">>> Unable to locate the CSCHitsValAlg Histogram service" << endreq;
  }
   
  return m_rootsvc;
} 



