#include "MuonHitTest/TGCHitsValAlg.h"
#include "EventInfo/EventInfo.h"
#include "EventInfo/EventID.h" 

#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/AlgFactory.h"

#include "Identifier/Identifier.h"

#include "GeoAdaptors/GeoMuonHits.h"

#include "MuonGeoModel/MuonDetectorManager.h" 
#include "MuonGeoModel/TgcReadoutElement.h"
 
#include "MuonSimEvent/TGCSimHitCollection.h"
#include "MuonSimEvent/TGCSimHit.h"

#include "MuonSimEvent/TgcHitIdHelper.h" 
#include "MuonIdHelpers/TgcIdHelper.h"

#include "GeneratorObjects/McEventCollection.h"
#include "CLHEP/Vector/LorentzVector.h"

#include <TH2D.h>
#include "TTree.h"
#include <string>

using namespace MuonGM;

void bookTGCHistograms();
void tgcbookTree();

TGCHitsValAlg::TGCHitsValAlg(const std::string& name, ISvcLocator* pSvcLocator) :
  Algorithm(name, pSvcLocator),
  m_pMuonMgr           (0),
  m_pTgcIdHelper       (0),				
  ValAlgInterface(), 
  log( msgSvc(), name )
{
  // Set users' request
  /**Properties to be modified through the jobOption file
     Short description:
     Key for accessing MC truth
     Option to write out ntuple
     AANtuple file (linked to a specific file name in the jobOption)
     AANtuple directory name for RPC variables
     AANtuple tree name for RPC variables
     If true, enter the main algorithm loop
  */
  declareProperty("McEventKey",     m_key="TruthEvent");
  declareProperty("WriteNtuple",          m_writeNtuple = true);
  declareProperty("NtupleFileName",       m_ntupleFileName = "MuonHits");
  declareProperty("NtupleDirectoryName",  m_ntupleDirName = "TGCHitsValidation");
  declareProperty("NtupleTreeName",       m_ntupleTreeName = "TGCTestHits");
  declareProperty("DoTGCTest",m_DoTGCTest=true);    
    	
  log << MSG::INFO << "++++++++++++  TGCHitsValAlg created  ++++++++++++"
      << endreq;
}

TGCHitsValAlg::~TGCHitsValAlg()
{
  log << MSG::INFO << " deleting TGCHitsValAlg " << endreq;
}

StatusCode TGCHitsValAlg::initialize()
{
  log << MSG::INFO << " initialize " << endreq;
  StatusCode status;

  /**Locate the StoreGateSvc and initialize our local ptr
     intitialize transient event store
  */
  status = service("StoreGateSvc", m_sgSvc);

  if (status.isFailure()) {
    log << MSG::FATAL << "TGCHitsValAlg: Could not retrieve StoreGateSvc!" << endreq;
    return status;
  }
  log << MSG::DEBUG << "Retrieved StoreGateSvc." << endreq;
          			
  StoreGateSvc* detStore = 0;
  status = service( "DetectorStore", detStore );
  if ( status.isSuccess() ) 
    {
      status = detStore->retrieve( m_pMuonMgr );
      if ( status.isFailure() ) 
	{
	  log << MSG::ERROR 
	      << " TGCHitsValAlg::initialize() :  Could not find MuonDetDescrMgr " 
	      << endreq;
	} 
      else 
	{ 
	  m_pTgcIdHelper = m_pMuonMgr->tgcIdHelper();
	}
    } 
  else 
    {
      log << MSG::ERROR 
	  << " TGCHitsValAlg::initialize() : MuonDetDescrMgr not found in DetectorStore " 
	  << endreq;
    }    
      
      
  if (status.isFailure()){
    log <<MSG::ERROR << "Could not get DetectorStore"<<endreq;
    return( StatusCode::FAILURE );
  }
 
  bookTGCHistograms();

  /// ROOT AANtuple Initialization---------------------------------------

  if (m_writeNtuple) {

    std::string tgcStreamAndPath = "/"+m_ntupleFileName+"/"+m_ntupleDirName;
    std::string tgctreeName = m_ntupleTreeName;
 
    std::string tgctreePath = tgcStreamAndPath+"/"+tgctreeName;

    tgctree = new TTree(TString(tgctreeName), "Muon TGC Hits output");
       
    StatusCode status;
    status=ToolRootHistSvc()->regTree(tgctreePath, tgctree);

    if(status.isFailure()) {
      log << MSG::DEBUG << "TGCHitsValAlg:: Unable to register TTreeTuple : " <<
	tgctreePath << endreq;
      return status;   
    }

  
    //-----------------
    /**add items-variables to the validation AANTUPLE
       Short description:
       Total number of G4 hits in AANTUPLE
       Total number of events processed
       Type 1=Hits , 2=Digits , 3=Prepraw
       Run number, event specific ID
       Global X, Y, Z, R of hit (with respect to the interaction point)
       TGC station name (char or int), conventional ATLAS naming scheme. Range: [T1E, T2E, T3E, T4E, T1F, T2F, T3F, T4F]
       TGC station eta, increases with Z. Range: [-5,5] ([-5,-1] backward endcap with -1 at lowest R, [1,5] forward endcap with 1 at lowest R)
       TGC station phi, increases with phi. Range: [1,48] ([1,48] for endcap, [1,24] for forward)
       TGC GasGap, the gas volume within a TGC chamber, increases with |Z|. Range: [1,3] ([1,2] for Doublets, [1,3] for Triplets)
       TGC Doublet MeasuresPhi, info not available in G4 hits!
       TGC IsStrip, info not available in G4 hits!	
    */

    tgctree->Branch("m_Validation_TGC_NumberOfHits",  &m_Validation_TGC_NumberOfHits, "m_Validation_TGC_NumberOfHits/I");
    tgctree->Branch("m_Validation_TGC_EventNumber",  &m_Validation_TGC_EventNumber, "m_Validation_TGC_EventNumber/I");
    tgctree->Branch("m_Validation_TGC_Type",  &m_Validation_TGC_Type, "m_Validation_TGC_Type/I");
    tgctree->Branch("m_Validation_TGC_RunNumber",  &m_Validation_TGC_RunNumber, "m_Validation_TGC_RunNumber/I");	 
    tgctree->Branch("m_Validation_TGC_HitX",  &m_Validation_TGC_HitX, "m_Validation_TGC_HitX/D"); 
    tgctree->Branch("m_Validation_TGC_HitY",  &m_Validation_TGC_HitY, "m_Validation_TGC_HitY/D"); 
    tgctree->Branch("m_Validation_TGC_HitZ",  &m_Validation_TGC_HitZ, "m_Validation_TGC_HitZ/D"); 
    tgctree->Branch("m_Validation_TGC_HitR",  &m_Validation_TGC_HitR, "m_Validation_TGC_HitR/D");
 
    tgctree->Branch("m_Validation_TGC_StName",  &m_Validation_TGC_StName, "m_Validation_TGC_StName/C");
    tgctree->Branch("m_Validation_TGC_StationEta",  &m_Validation_TGC_StationEta, "m_Validation_TGC_StationEta/I"); 
    tgctree->Branch("m_Validation_TGC_StationPhi",  &m_Validation_TGC_StationPhi, "m_Validation_TGC_StationPhi/I"); 
    tgctree->Branch("m_Validation_TGC_GasGap",  &m_Validation_TGC_GasGap, "m_Validation_TGC_GasGap/I");
    tgctree->Branch("m_Validation_TGC_IsStrip",  &m_Validation_TGC_IsStrip, "m_Validation_TGC_IsStrip/I");
    tgctree->Branch("m_Validation_TGC_Channel",  &m_Validation_TGC_Channel, "m_Validation_TGC_Channel/I");
 
 
  }

  log << MSG::INFO << "Initialisation ended     " << endreq;
  return StatusCode::SUCCESS;

} 

StatusCode TGCHitsValAlg::finalize()
{
  log << MSG::INFO << " finalize " << endreq;
	
  return StatusCode::SUCCESS;
}


/// Execute - on event by event

StatusCode TGCHitsValAlg::execute()
{

  log << MSG::INFO << " execute " << endreq;
  StatusCode sc;
	
  // ---------------------------------------
  // fill event data
  /** get EventInfo, used to obtain run and event number */
  const EventInfo* pevt;

  if (storeGateSvc()->retrieve(pevt).isFailure()) {
    log << MSG::FATAL << "Could not find event" << endreq;
    return(StatusCode::FAILURE);
  }
  else {
    log << MSG::INFO << "Found EventInfo in SG" << endreq;
  }

  if(isVerbose()) 
    {
      log << MSG::VERBOSE <<"Processing EventInfo event #"<<pevt->event_ID()->event_number() << " run: " << pevt->event_ID()->run_number() << endreq;
    }


  /**Get number of events processed and run number*/
  int evt = pevt->event_ID()->event_number();
  int numrun = pevt->event_ID()->run_number();
  m_Validation_TGC_RunNumber = pevt->event_ID()->run_number();	

  /**Access MC truth information*/
  const DataHandle<McEventCollection> mcEvent;
  
  if(storeGateSvc()->retrieve(mcEvent,m_key).isFailure())
    {
      log << MSG::FATAL << "TGCHitsValAlg: Could not find MCevent" << endreq;
      return StatusCode::SUCCESS;
    }
  else {
    log << MSG::INFO << "TGCHitsValAlg: Found MCEvent" << endreq;
  }	   
  

  DataVector<HepMC::GenEvent>::const_iterator e;
  if (mcEvent->size()!=1) return StatusCode::SUCCESS;
 
  HepVector3D direction;
  for (e=mcEvent->begin();e!=mcEvent->end(); e++) {
    for (HepMC::GenEvent::particle_const_iterator p= (**e).particles_begin(); p!= (**e).particles_end(); p++) 
      {
	direction.setX((**p).momentum().px());
	direction.setY((**p).momentum().py());
	direction.setZ((**p).momentum().pz());
	direction = direction.unit();
      }	
  }




  /// ROOT histograms ---------------------------------------	
  //MuonSpectrometer	
  /**Get the generic MuonSpectrometer (MDT,RPC,CSC,TGC) histograms
     Short description:
     MuonSpectrometer cross section
     MuonSpectrometer longitudinal 
     Number of events processed from truth
     Event specific run number
     Eta, Theta, Phi, z residual (...), phi residual (...) of hits in MuonSpectrometer
  */			
  TH2 *muondet;
  TH2 *muonlong_view;
  TH1 *event;
  TH1 *run;
  TH1 *eta;
  TH1 *theta;
  TH1 *phi;
  TH1 *zResid;
  TH1 *phiResid;	
  sc=tHistSvc()->getHist("/muonhitpos/muondet_barrel",muondet);
  sc=tHistSvc()->getHist("/muonhitpos/muonlongitudinal",muonlong_view);

  sc=tHistSvc()->getHist("/truth/event_num",event);
  sc=tHistSvc()->getHist("/truth/run_num",run);
	
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_eta",eta);
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_theta",theta);
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_phi",phi);
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_zResid",zResid);
  sc=tHistSvc()->getHist("/muonhitpos/muonhitpos_phiResid",phiResid);

	
  //TGC
  /**Get the TGC histograms
     Short description:
     MuonSpectrometer TGC cross section 
     MuonSpectrometer TGC longitudinal 
     Eta, Theta, Phi, z residual (...), phi residual (...) of hits in TGCs
  */
  TH2 *tgcdet;
  TH2 *tgclong_view; 
  TH1 *tgctheta;
  TH1 *tgceta;
  TH1 *tgcphi;
  TH1 *tgczResid;
  TH1 *tgcphiResid;
		
  sc=tHistSvc()->getHist("/tgchitpos/tgchitpos_eta",tgceta);
  sc=tHistSvc()->getHist("/tgchitpos/tgchitpos_theta",tgctheta);
  sc=tHistSvc()->getHist("/tgchitpos/tgchitpos_phi",tgcphi);
  sc=tHistSvc()->getHist("/tgchitpos/tgchitpos_zResid",tgczResid);
  sc=tHistSvc()->getHist("/tgchitpos/tgchitpos_phiResid",tgcphiResid);
  sc=tHistSvc()->getHist("/tgchitpos/tgcdet_barrel",tgcdet);
  sc=tHistSvc()->getHist("/tgchitpos/tgclongitudinal",tgclong_view);

  /**Enter the main algorithm loop*/
  if (m_DoTGCTest) {

				
    const DataHandle<TGCSimHitCollection> p_collection; 
    if (storeGateSvc()->retrieve(p_collection,"TGC_Hits")==StatusCode::FAILURE) 
      {
	log << MSG::FATAL << "No TGCSimHitCollection in StoreGate!" << endreq;
	return StatusCode::FAILURE;
      }
    else if(storeGateSvc()->retrieve(p_collection,"TGC_Hits")==StatusCode::SUCCESS)
      {

	/**Enter RPC hits loop, initialize the hits counter*/	
	counter_ValHitNumber =0 ;
	for(TGCSimHitCollection::const_iterator i_hit=p_collection->begin(); i_hit!=p_collection->end(); ++i_hit) 
	  {

	    /**Increment hits counter*/
	    ++counter_ValHitNumber;
		

	    /** Check the Hits identifiers, access the functions that give:
		Station name, station eta, station phi, doublet Z, doublet phi, doublet R, GasGap, Measures Phi.
		The values of these variables are written out to the AANtuple (variable content and range explained in the code section where AANTuple variables are
		registered)		
	    */			
	    HitID tgchit= (*i_hit)->TGCid();	          
	    TgcHitIdHelper* tgchelper = TgcHitIdHelper::GetHelper();
	    std::string tgc_stname = tgchelper->GetStationName(tgchit);
	    int         tgc_steta  = tgchelper->GetStationEta(tgchit);
	    int         tgc_stphi  = tgchelper->GetStationPhi(tgchit);
	    int         tgc_gg     = tgchelper->GetGasGap(tgchit); 
	    int         tgc_strip  = 1;//tgchelper->GetIsStrip(tgchit);
	    int         tgc_channel = 1;//tgchelper->GetChannel(tgchit);
     
 
	    /**Create the offline identifiers, fill them with hit info.Ready to be accessed and retrieve info.
	       Currently not used in this code except for some checks.
	    */
		            
	    Identifier offid = m_pTgcIdHelper->channelID(tgc_stname, tgc_steta, tgc_stphi, tgc_gg, tgc_strip, tgc_channel);
	    if (offid == 0)
	      {
		log << MSG::FATAL << "TGC: Cannot build a valid Identifier; skip " << endreq;
	      }
	    log << MSG::INFO << " TGC: Offline Id "<<m_pTgcIdHelper->show_to_string(offid) << endreq;
    
	    const MuonGM::TgcReadoutElement* descriptor = m_pMuonMgr->getTgcReadoutElement(offid);
	    if (descriptor != NULL)
	      {
		HepPoint3D xU;
		HepPoint3D localHit((*i_hit)->localPosition().x(),(*i_hit)->localPosition().y(),(*i_hit)->localPosition().z());
		xU = descriptor->localToGlobalCoords(localHit,offid);
	      }
	    else
	      {
		log << MSG::FATAL << "TGC readout element not found for Id = " << m_pTgcIdHelper->show_to_string(offid) << endreq;	 
		HepPoint3D(0,0,0);     
	      }
	    // -------- Testing Sim to Offline ID conversion ---------
     
	    HepPoint3D tgc_gPos   = descriptor->channelPos(offid);
	    double tgc_strip_phi   = tgc_gPos.phi();
	    if (tgc_strip_phi < 0.) tgc_strip_phi += 2.*M_PI;
	    double tgc_strip_perp  = tgc_gPos.perp();
	    double tgc_strip_z   = tgc_gPos.z();
	    double tgc_strip_cot   = 1./tan(tgc_gPos.theta());
		 
	    //Check Hits 
	    /**For every hit within the event, get the global position HepPoint3D u and then retrieve all releveant info
	       either from the HepPoint3D or from the MC vector (direction)
	    */
	    GeoTGCHit ghit(**i_hit);
	    if (!ghit) continue;	
	    HepPoint3D u = ghit.getGlobalPosition();
		
	    event->Fill(evt);
	    run->Fill(numrun);
		
	    muondet->Fill(u.x(),u.y());
	    tgcdet->Fill(u.x(),u.y());		
	    double rad=sqrt(u.x()*u.x()+u.y()*u.y());
		
	    muonlong_view->Fill(u.z(),rad);
	    tgclong_view->Fill(u.z(),rad);
		
	    theta->Fill(direction.theta());
	    tgctheta->Fill(direction.theta());

	    eta->Fill(direction.eta());
	    tgceta->Fill(direction.eta());
				
	    phi->Fill(direction.phi());
	    tgcphi->Fill(direction.phi());
		
	    zResid->Fill(u.cross(direction).dot(direction.cross(HepVector3D(0,0,1)).unit()));
	    phiResid->Fill(u.cross(direction).z());

	    tgczResid->Fill(u.cross(direction).dot(direction.cross(HepVector3D(0,0,1)).unit()));
	    tgcphiResid->Fill(u.cross(direction).z()); 		
		

	    /**AANtuple variables
	       Fill the AANtuple variables with a value obtained for each hit within the event
	    */
	    m_Validation_TGC_Type = 1; //(1: Hit, 2: Digit, 3: Reco)
	    m_Validation_TGC_NumberOfHits = counter_ValHitNumber;
	    m_Validation_TGC_EventNumber = evt;
	    m_Validation_TGC_HitX = u.x();
	    m_Validation_TGC_HitY = u.y();
	    m_Validation_TGC_HitZ = u.z();
	    m_Validation_TGC_HitR = rad;
		
	    strcpy(m_Validation_TGC_StName,(tgc_stname).c_str());  // put station name string in AANTUPLE as Val_StName  
	    m_Validation_TGC_StationEta = tgc_steta;
	    m_Validation_TGC_StationPhi = tgc_stphi;		
	    m_Validation_TGC_GasGap = tgc_gg;
	    m_Validation_TGC_IsStrip = tgc_strip;
	    m_Validation_TGC_Channel = tgc_channel;

	    /**After proccessing each hit, dump the info in the tree*/
	    tgctree->Fill();		
	  }/**Close the TGCSimHitCollection loop*/
	
    
      }/**Close part where TGCSimHitCollection is accessed from the StoreGate*/	
    
  }/**Close part of m_DoRPCTest*/

  return StatusCode::SUCCESS;
	
}/**Close execute*/
/*---------------------------------------------------------*/
ITHistSvc* TGCHitsValAlg::ToolRootHistSvc()
  /*---------------------------------------------------------*/
{
  MsgStream log(msgSvc(), name());
  
  StatusCode sc = service("THistSvc",m_rootsvc, true);
  if( sc.isFailure() ) {
    log << MSG::WARNING << ">>> Unable to locate the TGCHitsValAlg Histogram service" << endreq;
  }
   
  return m_rootsvc;
} 



